# (c) Copyright 2012-2013, CodeWeavers, Inc.

import os
import traceback

import gobject
gobject.threads_init()
import gtk

import pango

import bottlequery
import bottletreeview
import cxlog
import cxmenu
import cxutils

import bottlecollection
import cxguitools


import cxprefs
import cxsetup
import importdlg
import runcommanddlg
import installwizard
import cxregister
import distversion

from cxutils import cxgettext as _

EMPTY_LIST_NO_BOTTLES = "empty_nobottles"
EMPTY_LIST_NO_SHORTCUTS = "empty_noshortcuts"
EMPTY_LIST_SHORTCUTS_DISABLED = "empty_shortcuts_disabled"

class CrossOver(object):
    """This is the CrossOver application"""

    ccenter = None

    def __init__(self):

        # Load & show the main CrossOver window

        self.bottles = bottlecollection.sharedCollection(('basic',))
        self.bottles.addChangeDelegate(self)

        self.mainWindowXML = gtk.Builder()
        self.mainWindowXML.set_translation_domain("crossover")
        self.mainWindowXML.add_from_file(cxguitools.get_ui_path("crossover_mainwindow"))
        self.mainWindowXML.connect_signals(self)

        self.mainWindow = self.mainWindowXML.get_object("MainWindow")

        self.aboutDialog = None

        # do some fussing with the window layout before drawing.

        iconImage = self.mainWindowXML.get_object("CrossOverIconImage")
        logoImage = self.mainWindowXML.get_object("CrossOverLogoImage")

        iconImage.set_from_file(os.path.join(cxutils.CX_ROOT,
                                             "share",
                                             "images",
                                             "welcomeCrossOverIcon.png"))

        logoImage.set_from_file(os.path.join(cxutils.CX_ROOT,
                                             "share",
                                             "images",
                                             "welcomeCrossOverLogo.png"))

        self.updateAppList()
        self.mainWindow.show_all()
        self.buildMenuBar()


    def buildMenuBar(self):
        # This is a hack to work around a shortcoming in gtk.Builder < 2.16
        # There is no support for building menubar widgets from xml in
        # gtk.Builder prior to 2.16.  The widgets themselves are supported,
        # but you have to build them from code or continue using libglade.
        # If we ever stop targeting 2.10, we can put this in xml with the
        # rest of the UI.
        self.mainMenuBar = gtk.MenuBar()

        menu = gtk.Menu()
        menuItem = gtk.MenuItem(_("Quit"))
        menuItem.show()
        menuItem.connect("activate", self.quit_crossover_requested)
        menu.append(menuItem)

        rootMenuItem = gtk.MenuItem(_("CrossOver"))
        rootMenuItem.show()
        rootMenuItem.set_submenu(menu)
        self.mainMenuBar.append(rootMenuItem)

        menu = gtk.Menu()
        menuItem = gtk.MenuItem(_("Preferences"))
        menuItem.show()
        menuItem.connect("activate", self.on_PreferencesMenuItem_activate)
        menu.append(menuItem)

        rootMenuItem = gtk.MenuItem(_("Edit"))
        rootMenuItem.show()
        rootMenuItem.set_submenu(menu)
        self.mainMenuBar.append(rootMenuItem)

        menu = gtk.Menu()
        menuItem = gtk.MenuItem(_(u"Install Software\u2026"))
        menuItem.show()
        menuItem.connect("activate", self.on_InstallWindowsSoftware_clicked)
        menu.append(menuItem)

        menuItem = gtk.MenuItem(_(u"Manage Bottles\u2026"))
        menuItem.show()
        menuItem.connect("activate", self.on_ManageBottlesMenuItem_activate)
        menu.append(menuItem)

        menuItem = gtk.MenuItem(_(u"Run Command\u2026"))
        menuItem.show()
        menuItem.connect("activate", self.on_RunCommandMenuItem_activate)
        menu.append(menuItem)

        menuItem = gtk.SeparatorMenuItem()
        menuItem.show()
        menu.append(menuItem)

        menuItem = gtk.MenuItem(_(u"Migrate CrossOver Games Bottles\u2026"))
        menuItem.show()
        menuItem.connect("activate", self.on_ImportFromGamesMenuItem_activate)
        menu.append(menuItem)

        rootMenuItem = gtk.MenuItem(_("Tools"))
        rootMenuItem.show()
        rootMenuItem.set_submenu(menu)
        self.mainMenuBar.append(rootMenuItem)

        menu = gtk.Menu()
        menuItem = gtk.MenuItem(_("CrossOver Help"))
        menuItem.show()
        menuItem.connect("activate", self.on_ShowHelp_activate)
        menu.append(menuItem)

        menuItem = gtk.MenuItem(_(u"Register CrossOver\u2026"))
        menuItem.show()
        menuItem.connect("activate", self.on_DoRegistration_activate)
        menu.append(menuItem)

        menuItem = gtk.MenuItem(_("About"))
        menuItem.show()
        menuItem.connect("activate", self.on_ShowAboutCrossOver_activate)
        menu.append(menuItem)

        rootMenuItem = gtk.MenuItem(_("Help"))
        rootMenuItem.show()
        rootMenuItem.set_submenu(menu)
        self.mainMenuBar.append(rootMenuItem)

        mainWindowVBox = self.mainWindowXML.get_object("MainWindowVBox")
        mainWindowVBox.pack_start(self.mainMenuBar, False, False, 0)
        mainWindowVBox.reorder_child(self.mainMenuBar, 0)
        self.mainMenuBar.show()

    def updateAppList(self):
        windowsSoftwareScrolledWindow = self.mainWindowXML.get_object("WindowsSoftwareScrolledWindow")
        softwareListColumnVBox = self.mainWindowXML.get_object("SoftwareListColumnVBox")
        emptyReason = None

        emptyReason, windowsSoftwareList = self._getInstalledWindowsSoftware()


        # clear things out before we refresh
        for child in windowsSoftwareScrolledWindow.get_children():
            windowsSoftwareScrolledWindow.remove(child)

        for child in softwareListColumnVBox.get_children():
            if windowsSoftwareList:
                if child is not windowsSoftwareScrolledWindow:
                    softwareListColumnVBox.remove(child)
            else:
                softwareListColumnVBox.remove(child)


        if windowsSoftwareList:
            self._populateWindowsSoftwareWindow(windowsSoftwareScrolledWindow,
                                                windowsSoftwareList,
                                                softwareListColumnVBox)
        else:
            self._createEmptyWindowsSoftwareWindow(softwareListColumnVBox,
                                                   emptyReason)
            windowsSoftwareScrolledWindow.set_shadow_type(gtk.SHADOW_NONE)

        self.mainWindow.show_all()

    def bottleCollectionChanged(self):
        self.updateAppList()

    def on_ShowHelp_activate(self, _unused):
        # pylint: disable=R0201
        cxguitools.show_help("index.html")

    def on_ShowAboutCrossOver_activate(self, _caller):
        if self.aboutDialog is None:
            versionString = "%s %s" % (distversion.PRODUCT_NAME,
                                       distversion.PUBLIC_VERSION)
            self.aboutDialog = gtk.Dialog(title=_("About CrossOver"),
                                          parent=self.mainWindow,
                                          flags=gtk.DIALOG_NO_SEPARATOR | gtk.DIALOG_DESTROY_WITH_PARENT,

                                          buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))

            versionLabel = gtk.Label(versionString)
            versionLabel.show()
            crossOverIcon = gtk.Image()
            crossOverIcon.set_from_file(os.path.join(cxutils.CX_ROOT,
                                                     "share",
                                                     "images",
                                                     "welcomeCrossOverIcon.png"))
            crossOverIcon.show()

            codeWeaversLink = gtk.Label()
            codeWeaversLink.set_markup("<a href=\"http://www.codeweavers.com\">CodeWeavers, Inc.</a>")
            codeWeaversLink.show()

            # pylint is unable to figure out the
            # correct class of the dialog's vbox
            self.aboutDialog.vbox.pack_start(crossOverIcon) # pylint: disable=E1101
            self.aboutDialog.vbox.pack_start(versionLabel) # pylint: disable=E1101
            self.aboutDialog.vbox.pack_start(codeWeaversLink) # pylint: disable=E1101
            self.aboutDialog.vbox.show() # pylint: disable=E1101

            self.aboutDialog.connect("response", self.on_HideAboutCrossOver_activate)

        self.aboutDialog.present()

    def on_HideAboutCrossOver_activate(self, _unused, _unused_2):
        self.aboutDialog.destroy()
        self.aboutDialog = None

    def on_DoRegistration_activate(self, _unused):
        cxregister.open_or_show()

    def on_ManageBottlesMenuItem_activate(self, _unused):
        cxsetup.open_or_show()

    def on_RunCommandMenuItem_activate(self, _unused):
        runcommanddlg.open_or_show(None)

    def on_ImportFromGamesMenuItem_activate(self, _caller):
        importdlg.open_or_show(parent = self.mainWindowXML.get_object('MainWindow'))

    def on_PreferencesMenuItem_activate(self, _unused):
        cxprefs.open_or_show()

    def on_InstallWindowsSoftware_clicked(self, _unused):
        dlg = installwizard.open_or_show()
        dlg.add_observer(installwizard.INSTALL_FINISHED, self.installWizardDidFinishInstalling)

    def installWizardDidFinishInstalling(self, _event=None, _data=None, *_args):
        self.updateAppList()

    def quit_crossover_requested(self, _unused, _unused2=None):
        self.mainWindow.destroy()

    def on_destroy(self, _unused):
        self.bottles.removeChangeDelegate(self)
        cxguitools.toplevel_quit()

    def ccenterClosed(self):
        self.ccenter = None

    def on_seeAllWindowsApps_clicked(self, _unused):
        if not self.ccenter:
            self.ccenter = ControlCenter(self)

    def _getInstalledWindowsSoftware(self):
        # Returns a tuple including the reason the list of
        # Windows software is empty (if it is), and the list itself.
        #
        # The list we return is flattened across all bottles.
        # (this might not be appropriate for all purposes).

        installedApps = {}
        emptyReason = None
        hasShortcut = False
        hasEnabledShortcut = False

        bottleList = bottlequery.get_bottle_list()

        for bottle in bottleList:
            prefs = cxmenu.MenuPrefs(bottle, True)
            prefs.read_config()

            for app in prefs.data:
                hasShortcut = True
                key = bottle + "/" + app
                if prefs.data[app].include_in_mainmenu:
                    if prefs.data[app].include_in_mainmenu == 'alwaysinclude':
                        installedApps[key] = prefs.data[app]
                    elif prefs.data[app].include_in_mainmenu == 'neverinclude':
                        continue
                else:
                    installedApps[key] = prefs.data[app]
                    hasEnabledShortcut = True

        if not bottleList:
            emptyReason = EMPTY_LIST_NO_BOTTLES
        elif not hasShortcut:
            emptyReason = EMPTY_LIST_NO_SHORTCUTS
        elif not hasEnabledShortcut:
            emptyReason = EMPTY_LIST_SHORTCUTS_DISABLED

        return emptyReason, installedApps


    def _createEmptyWindowsSoftwareWindow(self, blankContainer, emptyReason):
        # Populates the windows software scrolled window with a simple
        # bit of text directing the user to install some windows apps
        # (or take other action, as appropriate).

        aVBox = gtk.VBox(False, 1)

        anHRule = gtk.HSeparator()
        aVBox.pack_start(anHRule)

        allAppsButton = None

        if emptyReason is None:
            raise ValueError("Must provide emptyReason when creating empty software window.\n")

        if emptyReason == EMPTY_LIST_NO_BOTTLES:
            pleaseInstallSoftwareLabel = gtk.Label(_("\n\n\n\nYou have not installed any\nWindows software yet.\n\nPlease click the button labeled\n'Install Windows Software'\nto get started."))
        elif emptyReason == EMPTY_LIST_NO_SHORTCUTS:
            pleaseInstallSoftwareLabel = gtk.Label(_("\n\n\n\nIt appears you have installed\nWindows applications, but no\nshortcuts have been created.\nSelect\n\nTools->Manage Bottles\n\nin CrossOver's menu to browse\nthe C: drive of each of\nyour bottles and find your Windows\napplications."))
        elif emptyReason == EMPTY_LIST_SHORTCUTS_DISABLED:
            pleaseInstallSoftwareLabel = gtk.Label(_("\n\n\n\nIt appears you have installed\nWindows software for use\nwith CrossOver, but none\nof your Windows shortcuts are\nconfigured to appear in\nCrossOver's main menu. Click\n\n'See All Windows Applications'\n\nto see all of your shortcuts."))
            allAppsButton = self.create_see_all_software_button()

        aVBox.pack_start(pleaseInstallSoftwareLabel)

        aVBox.set_child_packing(anHRule, False, True, 0, gtk.PACK_START)
        aVBox.set_child_packing(pleaseInstallSoftwareLabel, True, True, 0, gtk.PACK_START)

        if allAppsButton is not None:
            aVBox.pack_start(allAppsButton)
            aVBox.set_child_packing(allAppsButton, True, False, 0, gtk.PACK_END)

        blankContainer.pack_end(aVBox)



    def _populateWindowsSoftwareWindow(self,
                                       scrolledWindow,
                                       installedApps,
                                       softwareListColumn):

        seen_names = set()
        duplicate_names = set()
        for item in installedApps.values():
            name = item.menu_name()
            if name in seen_names:
                duplicate_names.add(name)
            else:
                seen_names.add(name)

        # Set up the scroll window so we can add items

        if scrolledWindow not in softwareListColumn.get_children():
            softwareListColumn.pack_start(scrolledWindow)

        windowsSoftwareTable = gtk.Table(4, 1, True)
        windowsSoftwareTable.set_homogeneous(True)
        windowsSoftwareTable.set_row_spacings(5)
        windowsSoftwareTable.set_col_spacings(5)

        scrolledWindow.add_with_viewport(windowsSoftwareTable)

        self.buttonDict = {}

        vpos = 0
        for app in sorted(installedApps.values(), cmp=installed_app_cmp):
            appName = app.menu_name()
            if appName in duplicate_names:
                tooltip_text = _("%(applicationName)s (in bottle: '%(bottleName)s')") % {'applicationName' : appName, 'bottleName' : app.parent.bottlename}
                aButton = self.createButton("\n%s\n" %
                                            appName,
                                            app.iconfile,
                                            tooltip_text)
            else:
                aButton = self.createButton("\n%s\n" %
                                            appName,
                                            app.iconfile)
            aButton.connect("button_press_event",
                            self.on_launchWindowsApp_clicked)
            self.buttonDict[aButton] = app
            windowsSoftwareTable.attach(aButton, 0, 1, vpos, vpos + 1,
                                        gtk.FILL, 0, 10)
            vpos += 1

        allAppsButton = self.create_see_all_software_button()
        softwareListColumn.pack_start(allAppsButton, False, True, 10)

    def create_see_all_software_button(self):
        aButton = gtk.Button()
        aButton.connect("clicked", self.on_seeAllWindowsApps_clicked)
        tmpLabel = gtk.Label(_("\nSee All Windows Applications\n"))
        attrs = pango.AttrList()
        attrs.change(pango.AttrWeight(pango.WEIGHT_ULTRABOLD))
        tmpLabel.set_attributes(attrs)
        myHBox = gtk.HBox()
        myHBox.pack_start(tmpLabel, True, True, 1)
        aButton.add(myHBox)
        return aButton

    def get_icon(self, icon_filename, height, width):
        icon = None
        if icon_filename:
            try:
                icon = gtk.gdk.pixbuf_new_from_file(icon_filename)
            except gobject.GError:
                cxlog.warn("couldn't load icon file %s:\n%s" % (cxlog.debug_str(icon_filename), traceback.format_exc()))
        if icon is None:
            icon = cxguitools.get_std_icon('crossover')
        if icon is not None:
            icon = icon.scale_simple(height, width, gtk.gdk.INTERP_BILINEAR)
        return icon


    def createButton(self, labelText, imagePath, tooltip_text=None):

        myButton = gtk.Button()
        myButtonBox = gtk.HBox()

        myLabel = gtk.Label(labelText)
        myLabel.set_alignment(0.0, 0.5)

        myImage = gtk.Image()
        myImage.set_from_pixbuf(self.get_icon(imagePath, 48, 48))

        myButtonBox.set_spacing(8)
        myButtonBox.pack_start(myImage, False, False, 1)
        myButtonBox.pack_start(myLabel, True, True, 1)

        myButton.add(myButtonBox)

        if tooltip_text is not None:
            tooltip = gtk.Tooltips()
            tooltip.set_tip(myButton, tooltip_text)

        return myButton

    def on_launchWindowsApp_clicked(self, buttonPressed, event):
        if event.button == 1:
            self.buttonDict[buttonPressed].start()
        elif event.button == 3:
            self.show_rightclick_menu(buttonPressed, event)

    def show_rightclick_menu(self, buttonToRemove, event):
        popupMenu = gtk.Menu()
        removeMenuItem = gtk.MenuItem(_("Remove from main menu"))
        removeMenuItem.show()
        removeMenuItem.connect("activate",
                               self.on_removebutton_activate,
                               buttonToRemove)
        popupMenu.append(removeMenuItem)
        popupMenu.show()
        popupMenu.popup(None, None, None, event.button, event.time)

    def on_removebutton_activate(self, _caller, buttonToRemove):
        self.buttonDict[buttonToRemove].make_neverinclude()
        self.updateAppList()

class ControlCenter(object):

    def __init__(self, creator):
        self.allAppsWindowXML = gtk.Builder()
        self.allAppsWindowXML.set_translation_domain("crossover")
        self.allAppsWindowXML.add_from_file(cxguitools.get_ui_path("allappswindow"))
        self.allAppsWindowXML.connect_signals(self)

        self.allAppsWindow = self.allAppsWindowXML.get_object("AllAppsWindow")
        self.allAppsWindow.show()

        self.application = creator

        # markup, sort, launcher
        self.launcherTreeStore = gtk.TreeStore(gtk.gdk.Pixbuf, str, str, object)
        self.launcherTreeStore.set_sort_column_id(1, gtk.SORT_ASCENDING)
        self.launcherTree = self.allAppsWindowXML.get_object('ProgramsTreeView')

        text_renderer = gtk.CellRendererText()
        img_renderer = gtk.CellRendererPixbuf()
        img_renderer.set_property("height", 20)
        img_renderer.set_property("width", 20)
        column = gtk.TreeViewColumn(_("Launcher"))
        column.pack_start(img_renderer, False)
        column.pack_start(text_renderer, False)
        column.add_attribute(img_renderer, 'pixbuf', 0)
        column.add_attribute(text_renderer, 'text', 1)

        self.launcherTree.append_column(column)

        self.launcherTree.set_model(self.launcherTreeStore)

        self.launcherTree.connect("button_press_event",
                                  self.on_ProgramsTreeView_buttonpress)

        self.launcherTree.connect("row_activated",
                                  self.on_ProgramsTreeView_activate)

        # markup, sort, bottlename
        self.bottleListStore = gtk.ListStore(str, str, object)
        self.bottleListStore.set_sort_column_id(1, gtk.SORT_ASCENDING)
        self.bottleList = self.allAppsWindowXML.get_object('BottlesList')

        renderer = gtk.CellRendererText()
        renderer.set_property('editable', True)
        column = gtk.TreeViewColumn(_("Bottle"), renderer, markup = 0)
        self.bottleList.append_column(column)

        self.bottleList.set_model(self.bottleListStore)

        self.bottles = bottlecollection.sharedCollection(('basic',))
        self.bottles.addChangeDelegate(self)

        self.updateBottleList()

        self.allAppsWindow.show()

    def closeControlCenter(self, _unused):
        self.allAppsWindow.destroy()
        self.bottles.removeChangeDelegate(self)
        self.application.ccenterClosed()

    def bottleCollectionChanged(self):
        self.updateBottleList()

    def updateBottleList(self):
        bottlesToAdd = set(self.bottles.bottleList())

        currentBottles = set()
        i = self.bottleListStore.get_iter_first()
        while i is not None:
            currentBottles.add(self.bottleListStore.get_value(i, 2))
            i = self.bottleListStore.iter_next(i)

        for bottle in bottlesToAdd - currentBottles:
            self.bottleListStore.append((cxutils.html_escape(bottle),
                                         bottle, bottle))

        for bottle in currentBottles - bottlesToAdd:
            i = self.bottleListStore.get_iter_first()
            while i is not None:
                if self.bottleListStore.get_value(i, 2) == bottle:
                    self.bottleListStore.remove(i)
                    break
                i = self.bottleListStore.iter_next(i)

        if bottlesToAdd:
            self.allAppsWindowXML.get_object('AllProgramsWindow').show()

    def get_selected_bottlename(self):
        cursor = self.bottleList.get_cursor()
        if cursor is None:
            return None

        path, _column = cursor

        return self.bottleListStore.get_value(self.bottleListStore.get_iter(path), 2)

    def add_launchers(self, items, parent):

        for launcher in items:
            desc = cxutils.html_escape(launcher.description)
            icon = None
            if launcher.menuitem is not None:
                fname = launcher.menuitem.iconfile
                icon = gtk.gdk.pixbuf_new_from_file(fname)
            elif launcher.children:
                icon = self.launcherTree.render_icon(gtk.STOCK_OPEN,
                                                     gtk.ICON_SIZE_SMALL_TOOLBAR)
            icon = icon.scale_simple(20, 20, gtk.gdk.INTERP_BILINEAR)

            row = self.launcherTreeStore.append(parent, (icon,
                                                         desc,
                                                         'x',
                                                         launcher))
            if launcher.children:
                self.add_launchers(launcher.children, row)
                if launcher.expanded:
                    self.launcherTree.expand_to_path(self.launcherTreeStore.get_path(row))

    def on_ProgramsTreeView_buttonpress(self, treeViewClicked, event):
        aPath, _col, _x, _y = treeViewClicked.get_path_at_pos(int(event.x),
                                                              int(event.y))
        myIter = self.launcherTreeStore.get_iter(aPath)
        aMenu = self.launcherTreeStore.get_value(myIter, 3).menuitem

        if aMenu is not None and event.button == 3:
            self.show_rightclick_menu(aMenu, event)

    def on_ProgramsTreeView_activate(self, _treeview, path, _viewcolumn):
        myIter = self.launcherTreeStore.get_iter(path)
        aMenu = self.launcherTreeStore.get_value(myIter, 3).menuitem

        if aMenu is not None:
            aMenu.start()

    def show_rightclick_menu(self, menuItem, event):

        rightClickMenu = gtk.Menu()

        addMenuItem = gtk.MenuItem(_("Add to main menu"))
        addMenuItem.show()
        addMenuItem.connect("activate",
                            self.on_addmentuitem_activate,
                            menuItem)

        removeMenuItem = gtk.MenuItem(_("Remove from main menu"))
        removeMenuItem.show()
        removeMenuItem.connect("activate",
                               self.on_removementuitem_activate,
                               menuItem)

        rightClickMenu.append(addMenuItem)
        rightClickMenu.append(removeMenuItem)
        rightClickMenu.show()
        rightClickMenu.popup(None, None, None, event.button, event.time)

    def on_addmentuitem_activate(self, _caller, menuToAdd):
        menuToAdd.make_alwaysinclude()
        self.application.updateAppList()

    def on_removementuitem_activate(self, _caller, menuToAdd):
        menuToAdd.make_neverinclude()
        self.application.updateAppList()

    def on_bottle_changed(self, _treeview):
        bottle = self.get_selected_bottlename()

        if bottle is None:
            return

        self.launcherTreeStore.clear()

        self.add_launchers(bottletreeview.BottleTreeView().get_items(bottle),
                           None)

def installed_app_cmp(a, b):
    result = cxutils.alpha_cmp(a.menu_name(), b.menu_name())
    if result == 0:
        result = cxutils.alpha_cmp(a.parent.bottlename, b.parent.bottlename)
    return result

