# (c) Copyright 2009-2013. CodeWeavers, Inc.

import gtk

import cxproduct
import cxobservable

import cxguitools

# These are the observable events
DIALOG_CLOSED = 'dialog_closed'


#
# Dialog instance
#

class PreferenceDialogController(cxobservable.Object):

    observable_events = frozenset((DIALOG_CLOSED,))

    def __init__(self):
        cxobservable.Object.__init__(self)

        #-----------------------------------------------------------------------
        #  Setup the GUI
        #-----------------------------------------------------------------------
        self.xml = gtk.Builder()
        self.xml.set_translation_domain("crossover")
        self.xml.add_from_file(cxguitools.get_ui_path("preferences"))
        self.xml.connect_signals(self)

        # Set initial values
        logUsage = False
        autoUpdate = False
        showUntested = False

        global_config = cxproduct.get_config()
        if global_config['CrossOver'].get('ReportWineUsage', '0') == '1':
            logUsage = True

        if global_config['OfficeSetup'].get('AutoUpdate', '0') == '1':
            autoUpdate = True

        if global_config['OfficeSetup'].get('ShowUntestedApps', '0') == '1':
            showUntested = True

        self.xml.get_object('usageCheckbox').set_active(logUsage)
        self.xml.get_object('downloadCheckbox').set_active(autoUpdate)
        self.xml.get_object('showUntestedCheckbox').set_active(showUntested)

    def downloadCheckbox_toggled_cb(self, caller):
        if caller.get_active():
            cxproduct.save_setting('OfficeSetup', 'AutoUpdate', '1')
        else:
            cxproduct.save_setting('OfficeSetup', 'AutoUpdate', '0')

    def usageCheckbox_toggled_cb(self, caller):
        if caller.get_active():
            cxproduct.save_setting('CrossOver', 'ReportWineUsage', '1')
        else:
            cxproduct.save_setting('CrossOver', 'ReportWineUsage', '0')

    def untestedCheckbox_toggled_cb(self, caller):
        if caller.get_active():
            cxproduct.save_setting('OfficeSetup', 'ShowUntestedApps', '1')
        else:
            cxproduct.save_setting('OfficeSetup', 'ShowUntestedApps', '0')

    def quit_requested(self, *_args):
        self.xml.get_object("PrefDlg").destroy()
        self.emit_event(DIALOG_CLOSED)
        cxguitools.toplevel_quit()

    def present(self):
        self.xml.get_object("PrefDlg").present()


#
# Global dialog management
#

DIALOG = None

def _dialog_closed(_event=None, _data=None, *_args):
    # pylint: disable=W0603
    global DIALOG
    DIALOG = None

def open_or_show(*args, **kwargs):
    # pylint: disable=W0603
    global DIALOG
    if DIALOG is None:
        DIALOG = PreferenceDialogController(*args, **kwargs)
        DIALOG.add_observer(DIALOG_CLOSED, _dialog_closed)
    else:
        DIALOG.present()
    return DIALOG
