# (c) Copyright 2011-2013. CodeWeavers, Inc.

import os
import time
import traceback

import gobject
gobject.threads_init()
import gtk

import cxproduct
import pyop
import cxlog
import cxutils
import cxobservable
import bottlequery
import bottlemanagement

import cxguitools

# for localization
from cxutils import cxgettext as _

_IMPORT = _("Move to CrossOver")
_COPY = _("Copy to CrossOver")
_IGNORE = _("Ignore")

# These are the observable events
DIALOG_CLOSED = 'dialog_closed'


#
# Dialog instance
#

class ImportDialogController(cxobservable.Object):

    observable_events = frozenset((DIALOG_CLOSED,))

    def __init__(self, selected_bottle=None, parent=None):
        cxobservable.Object.__init__(self)

        #----------------------------------------------------------------------
        #  Setup the GUI
        #----------------------------------------------------------------------
        self.xml = gtk.Builder()
        self.xml.set_translation_domain("crossover")
        self.xml.add_from_file(cxguitools.get_ui_path("importdlg"))
        self.xml.connect_signals(self)
        self.pulse_timer = None

        dlg_widget = self.xml.get_object("ImportBottlesDlg")
        self.parent = parent
        if self.parent:
            dlg_widget.set_transient_for(parent)
            dlg_widget.set_modal(True)
        else:
            self.xml.get_object("never_again_check").hide()

        try:
            icon = cxguitools.get_std_icon('cxrun')
            dlg_widget.set_icon(icon)
        except gobject.GError:
            cxlog.warn("Couldn't load icon:\n%s" % traceback.format_exc())


        #----------------------------------------------------------------------
        #  Set up progress bar
        #----------------------------------------------------------------------
        self.progbar = self.xml.get_object("ProgBar")
        self.progbar.set_pulse_step(.05)
        self.progbar.hide()

        #----------------------------------------------------------------------
        #  Set up bottle list
        #----------------------------------------------------------------------
        listviewwidget = self.xml.get_object("games_bottles_listview")
        if listviewwidget:
            renderer = gtk.CellRendererText()
            renderer.set_property('editable', False)
            column = gtk.TreeViewColumn(_("Bottle"), renderer, text = 0)
            column.set_resizable(True)
            listviewwidget.append_column(column)
            listviewwidget.get_column(0).set_sort_column_id(0)

            self.option_model = gtk.ListStore(gobject.TYPE_STRING)
            self.option_model.append((_IMPORT,))
            self.option_model.append((_COPY,))
            self.option_model.append((_IGNORE,))
            self.action_renderer = gtk.CellRendererCombo()
            self.action_renderer.set_property('has-entry', False)
            self.action_renderer.set_property('model', self.option_model)
            self.action_renderer.set_property('text-column', 0)
            self.action_renderer.set_property('editable', True)
            self.action_renderer.set_property('editable-set', True)
            self.action_renderer.connect('edited', self.on_action_changed)

            column = gtk.TreeViewColumn(_("Action"), self.action_renderer, text = 1)
            column.set_sort_column_id(1)
            listviewwidget.append_column(column)

            self.bottles_model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
            games_bottles_list = bottlequery.get_games_bottle_list()
            for bottle in games_bottles_list:
                newiter = self.bottles_model.append()
                self.bottles_model.set_value(newiter, 0, bottle)
                if selected_bottle and bottle != selected_bottle:
                    self.bottles_model.set_value(newiter, 1, _IGNORE)
                else:
                    self.bottles_model.set_value(newiter, 1, _IMPORT)
            listviewwidget.set_model(self.bottles_model)

        self.updating = False
        self.set_widget_sensitivities(self)
        dlg_widget.show()

    def present(self):
        self.xml.get_object("ImportBottlesDlg").present()

    def cancel_clicked(self, caller):
        if self.xml.get_object("never_again_check").get_active():
            cxproduct.save_setting('OfficeSetup', 'NeverImportGamesBottles', 'yes')
        self.xml.get_object("ImportBottlesDlg").destroy()
        self.quit_requested(caller)

    def quit_requested(self, _caller):
        self.emit_event(DIALOG_CLOSED)
        cxguitools.toplevel_quit()

    def progbar_pulse(self):
        self.progbar.pulse()
        return True

    def pulse_start(self, text):
        self.progbar.set_text(text)
        self.progbar.show()
        self.pulse_timer = gobject.timeout_add(100, self.progbar_pulse)

    def pulse_stop(self):
        if self.pulse_timer:
            gobject.source_remove(self.pulse_timer)
            self.pulse_timer = None
            self.progbar.hide()

    def on_action_changed(self, renderer, path, newtext):
        rowiter = self.bottles_model.get_iter(path)
        self.bottles_model.set_value(rowiter, 1, newtext)
        self.set_widget_sensitivities(renderer)

    def set_widget_sensitivities(self, _caller):
        if not self.updating:
            self.action_renderer.connect("edited", self.on_action_edited)
            if self.xml.get_object("never_again_check").get_active():
                self.xml.get_object("cancel_button").set_label(_("Cancel"))
            else:
                self.xml.get_object("cancel_button").set_label(_("Import Later"))

            importingAnything = False
            rowiter = self.bottles_model.get_iter_first()
            while rowiter:
                action = self.bottles_model.get_value(rowiter, 1)
                if action != _IGNORE:
                    importingAnything = True
                    break
                rowiter = self.bottles_model.iter_next(rowiter)

            self.xml.get_object("ok_button").set_sensitive(importingAnything)
        else:
            if hasattr(self.action_renderer, 'set_sensitive'):
                self.action_renderer.set_sensitive(False)
            self.pulse_start(_(u"Migrating bottles\u2026"))
            self.xml.get_object("games_bottles_listview").set_sensitive(False)
            self.xml.get_object("cancel_button").set_sensitive(False)
            self.xml.get_object("ok_button").set_sensitive(False)
            self.xml.get_object("never_again_check").hide()

    def on_action_edited(self, _cellrenderer, path, new_value):
        row_iter = self.bottles_model.get_iter(path)
        self.bottles_model.set_value(row_iter, 1, new_value)

    def migrate_clicked(self, _caller):
        self.updating = True
        self.set_widget_sensitivities(self)
        if self.xml.get_object("never_again_check").get_active():
            cxproduct.save_setting('OfficeSetup', 'NeverImportGamesBottles', 'yes')

        operation = ImportBottleOp(self.bottles_model, self)
        pyop.sharedOperationQueue.enqueue(operation)


#
# Global dialog management
#

DIALOG = None

def _dialog_closed(_event=None, _data=None, *_args):
    # pylint: disable=W0603
    global DIALOG
    DIALOG = None

def open_or_show(*args, **kwargs):
    # pylint: disable=W0603
    global DIALOG
    if DIALOG is None:
        DIALOG = ImportDialogController(*args, **kwargs)
        DIALOG.add_observer(DIALOG_CLOSED, _dialog_closed)
    else:
        DIALOG.present()
    return DIALOG


#
# Operations
#

class ImportBottleOp(pyop.PythonOperation):
    def __init__(self, bottles_model, parent_controller):
        pyop.PythonOperation.__init__(self)
        self.bottles_model = bottles_model
        self.parent_controller = parent_controller

    def find_games_root(self):
        product_dict = cxproduct.enum_products()
        use_this_root = None
        for cx_root in product_dict.keys():
            if product_dict[cx_root]['productid'] == "cxgames":
                use_this_root = cx_root
                break

        return use_this_root


    def shutdown_bottle(self, bottlename, gamesroot):

        #--------------------------------------------------------------
        #  Shuts down a bottle
        #  Returns false if the user cancels bottle shutdown.
        #  Otherwise, returns true
        #--------------------------------------------------------------
        if gamesroot:
            wine_command = os.path.join(gamesroot, "bin", "wine")
            args = [wine_command, "--bottle", bottlename, "--no-update",
                    "--ux-app", "wineserver", "-k0"]
            retcode = cxutils.system(args)
            if retcode == 0:
                cxreset_command = os.path.join(gamesroot, "bin", "cxreset")
                args = (cxreset_command, "--bottle", bottlename, "--noui", "--close")
                cxutils.run(args, stderr=cxutils.GRAB)

                time.sleep(12)

                # The return value from cxreset is not entirely reliable.
                # So now we check with wineserver -k0 to see if anything is
                # still running.
                wine_command = os.path.join(gamesroot, "bin", "wine")
                args = [wine_command, "--bottle", bottlename, "--no-update",
                        "--ux-app", "wineserver", "-k0"]
                retcode = cxutils.system(args)
                if retcode == 0:
                    cxlog.warn("User cancelled shutdown of %s." % bottlename)
                    return False
        return True


    def uninstall_bottle(self, bottlename, gamesroot):

        #--------------------------------------------------------------
        #  Shuts down a bottle and uninstalls its desktop integration.
        #  Returns false if the user cancels bottle shutdown.
        #  Otherwise, returns true
        #--------------------------------------------------------------


        #--------------------------------------------------------------
        # If we have no cxgames root, then cxgames is likely
        # already uninstalled.
        #
        # In that case, we don't need to shutdown bottles or
        # remove desktop integration (integration was already
        # removed during uninstall.)  If there are games bottles left,
        # those are from the user deliberately chosing to uninstall
        # the games product but to leave the games bottles, and we can
        # migrate those without further action.
        #--------------------------------------------------------------

        if gamesroot:
            retcode = self.shutdown_bottle(bottlename, gamesroot)
            if not retcode:
                # user has cancelled shutdown
                return False
            args = (os.path.join(gamesroot, "bin", "cxbottle"),
                    "--bottle", bottlename, "--removeall")
            retcode, _out, err = cxutils.run(args, stderr=cxutils.GRAB)
            if retcode != 0:
                cxlog.log(u"Failed to uninstall \'%s\' before migrating. Error: %s Forging ahead\u2026" % (bottlename, cxlog.to_str(err)))

        return True

    def main(self):
        rowiter = self.bottles_model.get_iter_first()
        gamesroot = self.find_games_root()
        while rowiter:
            name = self.bottles_model.get_value(rowiter, 0)
            action = self.bottles_model.get_value(rowiter, 1)
            if action == _COPY:
                self.parent_controller.pulse_start(_(u"Copying %s\u2026") % name)
                if self.shutdown_bottle(name, gamesroot):
                    bottlemanagement.copy_games_bottle(name)
            elif action == _IMPORT:
                self.parent_controller.pulse_start(_(u"Importing %s\u2026") % name)
                if self.uninstall_bottle(name, gamesroot):
                    bottlemanagement.migrate_games_bottle(name)
            rowiter = self.bottles_model.iter_next(rowiter)
            self.parent_controller.pulse_stop()


    def finish(self):
        self.parent_controller.xml.get_object("ImportBottlesDlg").destroy()
        self.parent_controller.quit_requested(self)

